from pyicubsim import iCubBall, iCubHead, iCubTorso, iCubRightArm, iCubLeftArm, iCubRightEye, iCubEmotion, coord, hit, reset, stop_iCubSim
import time
import os
import cv2
import signal

# exit on ctrl-c
def signal_handler(signal, frame):
    os._exit(0)

signal.signal(signal.SIGINT, signal_handler)

ball = iCubBall()
head = iCubHead()
torso = iCubTorso()
rightArm = iCubRightArm()
leftArm = iCubLeftArm()

reset([torso,rightArm])
time.sleep(2)
torso.set(joint2=20)
time.sleep(2)

goal = ball.get()
hit(torso,rightArm,goal)
print(goal,coord(torso,rightArm))

time.sleep(2)
reset([torso,rightArm])
time.sleep(2)

def pick_up():
    torso.set(joint0=-1, joint1=0.6, joint2=23.20)
    rightArm.set(joint0=-52.5, joint1=0, joint2=5)
    leftArm.set(joint0=-52.5, joint1=0, joint2=5)
    time.sleep(1.2)
    torso.set((0,0,0))

def set_standard_positions():
    standard_arm_position = (0, 80, 0, 50, 0, 0, 0, 59, 20, 20, 20, 10, 10, 10, 10, 10)
    rightArm.set(standard_arm_position)
    leftArm.set(standard_arm_position)
    head.set((-37.80, 0, -8.800, 0, 0, 0))
    torso.set(joint0=-1, joint1=0.6, joint2=23.20)
    time.sleep(1.5)
    
ball.set((-300,0,-46))
pick_up()
reset([rightArm,leftArm,torso])
time.sleep(2)
set_standard_positions()

def speak(text):
    print(text)
    filename = str(time.time())+'.txt'
    file = open(filename,'wb') 
    file.write(bytes(text,'utf-16')) 
    file.close() 
    os.environ["ESPEAK_DATA_PATH"] = "." 
    os.system('espeak -v SK -b4 -f '+filename)
    os.remove(filename)
    
speak("Ahoj Janko")

emotion = iCubEmotion()
emotion.set('hap')
"""
  neu (neutral)
  hap (happy)
  sad (sad)
  sur (surprised)
  ang (angry)
  evi (evil)
  shy (shy)
  cun (cunning)
"""

torso.set(joint2=0)
time.sleep(3)
torso.set(joint2=20)

def set_standard_positions():
    standard_hand_position = (0, 80, 0, 50, 0, 0, 0, 59, 20, 20, 20, 10, 10, 10, 10, 10)
    rightArm.set(standard_hand_position)
    leftArm.set(standard_hand_position)
    head.set((-37.80, 0, -8.800, 0, 0, 0))
    torso.set(joint0=-1, joint1=0.6, joint2=23.20)
    time.sleep(1.5)

def blue_filter(bgr):
    # converting from BGR to HSV color space
    hsv = cv2.cvtColor(bgr, cv2.COLOR_BGR2HSV)
    # Hue range for blue derived from its BGR
    blue = np.uint8([[[80, 0, 0]]])
    hsvBlue = cv2.cvtColor(blue,cv2.COLOR_BGR2HSV)
    lowerLimit = np.array([hsvBlue[0][0][0]-10,100,25],np.uint8)
    upperLimit = np.array([hsvBlue[0][0][0]+10,255,255],np.uint8)
    # filtering in HSV
    mask = cv2.inRange(hsv, lowerLimit, upperLimit)
    # redraw the detected ball
    indices = np.where(mask > 0)
    bgr[indices[0],indices[1],:] = [255,255,0]
    # if pixel number of ball is too small, it is perhaps not the ball at all
    if np.shape(indices)[1] < 500:
        # ball not found
        return None
    else:
        # return average of x and y coordinates of blue pixels
        return (int(np.average(indices[1])),int(np.average(indices[0])))

print('press Esc ...')
rightEye = iCubRightEye()
while True:
    frame = rightEye.grab()
    cv2.imshow('rightEye',frame)
    if cv2.waitKey(10) == 27:
        break

cv2.destroyAllWindows()

stop_iCubSim()

from agentspace import Agent,Space

class DemoAgent(Agent):
    def init(self):
        self.attach_timer(1)
    def senseSelectAct(self):
        print("writing to the blackboard")
        Space.write("data",0) # ,validity=0.5,priority=2
        d = Space.read("data",-1)

a = DemoAgent()
time.sleep(5)
a.stop()

print('press ctrl-c')
